Package: HVT
Type: Package
Date: 2025-02-03
Title: Constructing Hierarchical Voronoi Tessellations and Overlay
        Heatmaps for Data Analysis
Version: 25.2.2
Authors@R: c(
	person("Zubin", "Dowlaty", email = "zubin.dowlaty@mu-sigma.com", role = "aut"),
    	person("Mu Sigma, Inc.", email = "ird.experiencelab@mu-sigma.com", role = "cre"))
Description: Facilitates building topology preserving maps for data analysis.
License: Apache License 2.0
Encoding: UTF-8
Imports: MASS, grDevices, splancs, stats, dplyr, NbClust, purrr,
        magrittr, ggplot2, tidyr, scales, cluster, reshape2,
        FNN,Rtsne,umap, plyr, gganimate, markovchain, methods,deldir,
        gridExtra
Depends: R (>= 4.0.0)
BugReports: https://github.com/Mu-Sigma/HVT/issues
URL: https://github.com/Mu-Sigma/HVT
RoxygenNote: 7.3.2
Suggests: knitr,rmarkdown,testthat,geozoo,plotly,
        DT,patchwork,sp,Hmisc,data.table,gtable,
        htmlwidgets,installr,skimr,tibble,devtools,gifski,
        tidyverse,DataExplorer,htmltools,corrplot,kableExtra,polyclip,conf.design
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-02-05 09:15:45 UTC; vishwavani
Author: Zubin Dowlaty [aut],
  Mu Sigma, Inc. [cre]
Maintainer: "Mu Sigma, Inc." <ird.experiencelab@mu-sigma.com>
Repository: CRAN
Date/Publication: 2025-02-05 12:50:02 UTC
Built: R 4.3.3; ; 2025-02-15 14:42:53 UTC; unix
