# Generated by roxygen2: do not edit by hand

export(HVTMSMoptimization)
export(OptimizationResults)
export(clustHVT)
export(displayTable)
export(edaPlots)
export(getTransitionProbability)
export(msm)
export(plotAnimatedFlowmap)
export(plotExAnteRawSeries)
export(plotHVT)
export(plotModelDiagnostics)
export(plotMsmKN)
export(plotNovelCells)
export(plotQuantErrorHistogram)
export(plotStateTransition)
export(plotZscore)
export(reconcileTransitionProbability)
export(removeNovelty)
export(scoreHVT)
export(scoreLayeredHVT)
export(summary)
export(trainHVT)
import(dplyr)
import(ggplot2)
import(markovchain)
import(methods)
import(tidyr)
importFrom(dplyr,across)
importFrom(dplyr,between)
importFrom(dplyr,case_when)
importFrom(dplyr,everything)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_datetime)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,unit)
importFrom(magrittr,"%>%")
importFrom(plyr,rbind.fill)
importFrom(stats,as.dendrogram)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(tidyr,pivot_longer)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,tail)
